pragma solidity >=0.4.24 <0.6.11;

import "./Table.sol";

contract Order {
    event SetResult(int256 count);

    KVTableFactory tableFactory;
    string constant TABLE_NAME = "t_order";

    constructor() public {
        //The fixed address is 0x1010 for KVTableFactory
        tableFactory = KVTableFactory(0x1010);
        // the parameters of createTable are tableName,keyField,"vlaueFiled1,vlaueFiled2,vlaueFiled3,..."
        tableFactory.createTable(TABLE_NAME, "orderNumber", "orderInfo");
    }

    //get record
    function get(string memory orderNumber) public view returns (bool, string memory) {
        KVTable table = tableFactory.openTable(TABLE_NAME);
        bool ok = false;
        Entry entry;
        (ok, entry) = table.get(orderNumber);
        string memory orderInfo;
        if (ok) {
            orderInfo = entry.getString("orderInfo");
        }
        return (ok, orderInfo);
    }

    //set record
    function set(string memory orderNumber, string memory orderInfo)
    public
    returns (int256)
    {
        KVTable table = tableFactory.openTable(TABLE_NAME);
        Entry entry = table.newEntry();
        // the length of entry's field value should < 16MB
        entry.set("orderNumber", orderNumber);
        entry.set("orderInfo", orderInfo);
        // the first parameter length of set should <= 255B
        int256 count = table.set(orderNumber, entry);
        emit SetResult(count);
        return count;
    }
}
